
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/
#include "p18f2523.h"

#include <delays.h>

#include "lcd.h"


void delay(void) ;
void set_lcd_control(void);
void display_trigger(void);
void data_trigger(void) ;
void busy_test(void) ;

unsigned char lcd_pos;
char temp;
char the_letter;
unsigned char new_cursor;



void ResetDisplay(void) { 	// does not require 4 bit mode...
	set_lcd_control();
	delay();		//  Wait 15 ms
	delay();
	delay();
	delay();
	lcd_pos = 0;

	busy_test();		// we are now in 8 bit mode
	PORTB = 0x30;		//  and we only have to send once
	display_trigger();
	delay();		// Wait 4.1 ms
	delay();
	display_trigger();		//  Wait 100 us
	delay();
					// now we are in 4 bit mode
					// Now set function mode
	busy_test();		// we are now in 8 bit mode
	PORTB = 0x20;             //  to 4bit bit mode
	display_trigger();		// and to 2 line mode

}

void ClearDisplay(void) {
	busy_test();		/* Wait til not busy */
				/* 01 is the command */
	PORTB = 0x00;			/* Clear PORT b	*/
	display_trigger();		/* trigger the 4 bit interface */ 

	PORTB = 0x10;			/* Next nibble */
	display_trigger();		/* Trigger it again */
}

void HomeDisplay(void) {
	lcd_pos = 0;		/* For our memory	*/
	busy_test();		
	/* 02 is the command */
	PORTB = 0x00;		/* Upper half  */
	display_trigger();
	PORTB = 0x20;		/* Lower half  */
	display_trigger();
}

void EntryDisplay(void) {
	busy_test();
	/* 28 is the command  */
	PORTB = 0x20;		/* Upper half */
	display_trigger();
	PORTB = 0x80;		/* Lower half  */
	display_trigger();	
}

void ActivateDisplay(void) {
	busy_test();
		/* 0c is the command  */
	PORTB = 0x00;		/* Upper half */
	display_trigger();
	PORTB = 0xc0;		/* Lower half */
	display_trigger();
}

void CursorJump(unsigned char pos) {

	lcd_pos = pos;
	busy_test();
	/* Get the lower 3 bits of the positon	*/
	temp = lcd_pos & 0x70;
	temp = temp | 0x80;		/* Turn on the high bit */
	PORTB = temp;
	display_trigger();		/* and send it  */
	temp = lcd_pos << 4;	/* now get the lower bits */	
	PORTB = temp;
	display_trigger();		/* and send them */
}

void CursorBack(void) {
	lcd_pos--;
	busy_test();
		/* 10 is the command  */
	PORTB = 0x10;		/* Upper half  */
	display_trigger();
	PORTB = 0x00;		/* Lower half */
	display_trigger();
}

void CursorForward(void) {
	busy_test();
	/* 14 is the command  */
	PORTB = 0x10;		/* Upper half */
	display_trigger();
	PORTB = 0x40;		/* Lower half  */
	display_trigger();
}

void CursorGoto(unsigned char pos) {
	new_cursor = pos;
	HomeDisplay();		/* Atrt at the beginning */
	while (new_cursor) {	/*  and loop  */
		CursorForward();	 /* One by one */
		new_cursor--;		/* Until we get the the desire place  */
	}
}

void WriteCharacter(char ch) {
	busy_test();
	the_letter = ch;

	PORTB = ch & 0xf0;		/* Get the upper half  */
	data_trigger();		/* Trigger some data */
	the_letter = ch << 4;	/* Get the lower half  */
	
	PORTB = the_letter;
	data_trigger();		/* Trigger it again */
}

void WriteString(char * s) {
	while (*s != 0) {	/* repeat this 	*/
		WriteCharacter(*s);	/* for every char */
		s++;				/* to write a whole string  */
	}
}

/* This one does the same as the one above, 
	except it accepts a pointer to ROM (program memory) */
void WriteRom(static rom char * s, unsigned char pos) {
	CursorJump(pos);
	while (*s != 0) {
		WriteCharacter(*s);
		s++;
	}
}

/**********************************************
	Internal Use Functions
*********************************************/

void delay(void) {
	Delay10KTCYx(10);		/* Kinda explains itself... */
}


void set_lcd_control(void) {
	PORTBbits.RB1 = 0;		/* Set the Port bits */
	PORTBbits.RB2 = 0;
	PORTBbits.RB3 = 0;
	TRISB = 0x00;			/* and the direction bits */
}

void display_trigger(void) {	
	PORTBbits.RB1 = 1;		/* Turn it on... */
	Delay1KTCYx(5);		/* Kinda arbitrary, but it works */
	PORTBbits.RB1 = 0;		/* Turn it off */
}


void data_trigger(void) {
	PORTBbits.RB2 = 1;		/* Data (not commands) first  */	
	PORTBbits.RB1 = 1;		/* Next enable the module	*/
	Delay1KTCYx(5);
	PORTBbits.RB1 = 0;		/* Now back off */
	PORTBbits.RB2 = 0;		/* and done with the  message */ 
	return;
}



void busy_test(void) {
	TRISBbits.TRISB7 = 1;		/* Set the bigh bit as an input */
	PORTBbits.RB2 = 0;		/*  Set to 'command' */
	PORTBbits.RB3 = 1;		/*  Set to 'Read */
	PORTBbits.RB1 = 1;		/* and enable  */
	Delay1KTCYx(3);		/* settling time */
	while (PORTBbits.RB7) {	/*  Wait for the module... */
		Delay1KTCYx(1);		/*  to not be busy  */
	}
	PORTBbits.RB1 = 0;		/* And go back to the way we were */
	PORTBbits.RB3 = 0;
	TRISBbits.TRISB7 = 0;
}