
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/

#include "p18f2523.h"
#include <eep.h>
#include "extern.h"
#include "tools.h"


void WriteToEE(unsigned char addr, unsigned char val) {
	Busy_eep();
	Write_b_eep(addr,val);
}
static unsigned char value;
void ReportReset( ) {
	/* Determine the reason for the reset, and store it in the EEPROM.*/
	/* Although, there is no way for us to view our own resets at this time */
	if (!RCONbits.BOR) {
		value = Read_b_eep(BROWNOUT);
		value++;
		WriteToEE(BROWNOUT,value);
	}	
	if (!RCONbits.POR){
		value = Read_b_eep(POWERUP);
		value++;
		WriteToEE(POWERUP,value);
	}	 
	if (!RCONbits.PD) {

	}	
	if (!RCONbits.TO) {
		value = Read_b_eep(WATCHDOG);
		value++;
		WriteToEE(WATCHDOG,value);
	}	
	if (!RCONbits.RI) {
		value = Read_b_eep(RESET);
		value++;
		WriteToEE(RESET,value);
	}	
	RCONbits.BOR = 1;
	RCONbits.POR = 1;
	RCONbits.TO = 1;
	RCONbits.RI = 1;
	RCONbits.PD = 1;
}

/*  Pull out some bytes from ROM to RAM  */
void str2ram(static char *dest, static char rom *src) {
  while ((*dest++ = *src++) != '\0')
    ;
}
/* Simple little routine to convert INT to a two digit ASCII 
	Wrttien when space was at a premium, and didn't want to 
	import the library call, because of ROM space 
*/
void int2ascii(char * target, unsigned char val, unsigned char leading) {
	if (val >= 90) target[0] = '9';
	else if (val >= 80) target[0] = '8';
	else if (val >= 70) target[0] = '7';
	else if (val >= 60) target[0] = '6';
	else if (val >= 50) target[0] = '5';
	else if (val >= 40) target[0] = '4';
	else if (val >= 30) target[0] = '3';
	else if (val >= 20) target[0] = '2';
	else if (val >= 10) target[0] = '1';
	else {
		if (leading) target[0] = '0';
		else target[0] = ' ';
	}
	value = val % 10;
	target[1] = value + '0';
}
/* Set the timer to run on overflows from the 32,768 external OSC  */
void InitTimer1(void ) {
	TMR1H = 0x80;
	TMR1L = 0x00;
	sec = 50;		/* Start with an arbitrary time of 8:59 pm  */
	min = 59;
	hour = 20;
	T1CONbits.TMR1CS = 1;
	T1CONbits.T1SYNC = 1;
	T1CONbits.T1OSCEN = 1;
	T1CONbits.TMR1ON = 1;
	PIE1bits.TMR1IE = 1;
}

/* Initialize timer 2 to handle creating an audiable sound */
void InitializeAlarm(void) {
	T0CONbits.T0CS = 0;	// Clock = CPU clock
	T0CONbits.PSA = 0;		// Prescalar is assigned (backwards)
	T0CONbits.T0PS0 = 1;	//  Prescalar = 16
	T0CONbits.T0PS1 = 1;
	T0CONbits.T0PS2 = 0;
	T0CONbits.T08BIT = 1;	//  Run in 8 bit mode
	T0CONbits.TMR0ON = 0;	
	INTCONbits.TMR0IE = 0;

}

/* If we need to turn on the sound... */
void ActivateAlarm(unsigned char on ) {
	if (on) {		/* Enable the timer 2 and it's interrupts */
		T0CONbits.TMR0ON = 1;
		INTCONbits.TMR0IE = 1;
		machine |= ALARM;
	} else {	/* Otherwise, clear it, and set the output to low */
		T0CONbits.TMR0ON = 0;
		INTCONbits.TMR0IE = 0;
		INTCONbits.TMR0IF = 0;
		LATAbits.LATA1 = 0;
		machine &= ~ALARM;	/* And note the machine state */
	}

}

